package com.borcon.aop.application;

import com.borcon.aop.business.*;
import com.borcon.aop.security.*;

/**
 * Borcon 2002 : 2106 - Programmation oriente aspect
 * @author Emmanuel Boudrant, Olivier Mangez
 *
 * Pour plus d'information :
 *   http://www.mangez.info
 *   http://www.mycgiserver.com/~eboudrant/borcon2002/
 */

public class ForumLoader extends Exception {

  final static User SYSTEM = new User("_SYSTEM_");

  final static String aopMessage = "<p>&laquo; La programmation orient&eacute;e aspect pourrait bien \n    devenir la prochaine r&eacute;volution dans le monde du d&eacute;veloppement \n    logiciel. En effet, en &eacute;tendant la programmation orient&eacute;e objet \n    avec la notion d'aspect, elle permet de rendre modulaire des probl&egrave;mes \n    transverses tels que la mise en place d'un m&eacute;canisme de log ou la \n    gestion des droits. </p>\n  <p>Cette session sera l'occasion de pr&eacute;senter les \n    concepts introduits par la programmation orient&eacute;e aspect. Nous montrerons \n    ensuite comment on peut la mettre en oeuvre avec JBuilder et AspectJ - \n    une extension orient&eacute;e aspect de Java. Enfin, nous proposerons des exemples \n    d'application de cette technique parmi lesquels le design by contract. \n  &raquo;</p>\n";
  final static String dbcMessage = "<p>&laquo; La qualit&eacute; constitue s&ucirc;rement l'un \n  des enjeux majeurs du d&eacute;veloppement logiciel. Plusieurs techniques adapt&eacute;es \n  &agrave; la programmation orient&eacute;e objet ont &eacute;t&eacute; imagin&eacute;es \n  pour s'assurer de cette qualit&eacute;. Si l'eXtreme Programming a \n  popularis&eacute; l'utilisation des tests unitaires, Bertrand Meyer, quant \n  &agrave; lui, a propos&eacute; depuis longtemps la mise en oeuvre de la \n  notion de contrat. </p>\n<p>Cette technique - on l'appelle Design by contract \n  ou programmation par contrat - consiste &agrave; exprimer les sp&eacute;cifications \n  des composants logiciels sous la forme de contrats et &agrave; v&eacute;rifier \n  en permanence le respect de ces derniers.</p>\n<p>Apr&egrave;s avoir pr&eacute;sent&eacute; les principes du \n  Design by contract, nous nous int&eacute;resserons &agrave; sa mise en oeuvre \n  dans des langages comme le C++, Delphi ou Java."
                                 + "Par ailleurs, nous montrerons \n  pourquoi cette technique est compl&eacute;mentaire des tests unitaires pr&eacute;conis&eacute;s \n  par XP. &raquo;</p\n";
  final static String refMessage = "<p class=\"comment\">&laquo; L\'eXtreme Programming - connue aussi sous les \n    initiales XP - est une m&eacute;thode de d&eacute;veloppement originale, \n    n&eacute;e sous l'impulsion de Kent Beck. On aurait pu croire qu'il \n    ne s'agissait que d'un effet de mode ; on sait aujourd\'hui qu\'il n\'en \n    est rien. Pourquoi ? Parce qu'XP a su promouvoir des techniques aujourd'hui \n    indispensables : l'emploi syst&eacute;matique des tests unitaires, bien \n    s&ucirc;r, mais aussi le refactoring. </p>\n  <p class=\"comment\">Cette pratique consiste &agrave; remanier le code existant \n    dans un souci permanent de simplicit&eacute; et de qualit&eacute;. Elle permet \n    notamment, par des transformations microscopiques et successives, de faire appara&icirc;tre \n    une architecture dans du code qui en &eacute;tait d&eacute;pourvu.</p>\n  <p class=\"comment\">Au cours de cette session, nous pr&eacute;senterons la philosophie \n    du refactoring."
                                 + " Nous examinerons ensuite comment les environnements de d&eacute;veloppement \n    tels que JBuilder ont int&eacute;gr&eacute; cette pratique. Enfin, nous montrerons \n  des exemples concrets de sa mise en oeuvre. &raquo;</p>\n";

  final static String letMessage = "<p>Les architectures mises en place pour les applications web deviennent \n  de plus en plus compliqu&eacute;es et sont g&eacute;n&eacute;ralement \n  organis&eacute;es en une multitude de couches (jusqu\'&agrave; cinq parfois). \n  Par cons&eacute;quent, il devient maintenant n&eacute;cessaire d\'uniformiser \n  ces diff&eacute;rentes couches en mettant en oeuvre les Frameworks. D&eacute;velopp&eacute; \n  au sein de la communaut&eacute; Apache, le projet Struts propose ainsi \n  un framework web dont l\'objectif est de g&eacute;rer l\'interface \n  Homme-Machine d\'une application web. Ce framework, tr&egrave;s en \n  vogue, propose un paradigme Model-View-Controller 2. Il fournit des fondations \n  solides permettant au d&eacute;veloppeur de se concentrer avec plus d\'efficacit&eacute; \n  sur la partie m&eacute;tier. </span></p>\n<p >La premi&egrave;re partie de cette \n  session est consacr&eacute;e &agrave; la pr&eacute;sentation du framework \n  Struts, les concepts, les composants"
                                 + "et ses outils de productivit&eacute; \n  d&eacute;velopp&eacute;s par la communaut&eacute; Open-Source. La deuxi&egrave;me \n  partie se concentrera sur la mise en oeuvre d\'un projet Struts \n  avec JBuilder.&quot;<br>";
  final static String strMessage = "<p>A ce jour, il existe de nombreuses \n    strat&eacute;gies pour g&eacute;rer les diff&eacute;rents aspects, plus \n    ou moins complexes, d'un site dynamique. Le framework Struts est \n    tr&egrave;s suivi par la communaut&eacute; Open-Source. Le but de cette \n    session consiste pr&eacute;cis&eacute;ment &agrave; montrer comment tirer \n    le meilleur parti du framework Struts en utilisant : </p>\n  <ul>\n    <li class=\"normalnoir\">l'automatisation dans Struts,</li>\n    <li class=\"normalnoir\">les diff&eacute;rentes strat&eacute;gies et design \n      patterns existants,</li>\n    <li class=\"normalnoir\">les extensions autour de struts</li>\n    <li class=\"normalnoir\">les GUIs et autres g&eacute;n&eacute;rateurs de \n      code,</li>\n    <li class=\"normalnoir\">Struts et JUnit. (sous r&eacute;serve).</li>\n      </ul>";

  public ForumManager getForumManager() {
    ForumManager manager = new ForumManager();

    try {
      manager.addForum(createLetsStrutsItForum(), SYSTEM);
      manager.addForum(createRefactoringForum(), SYSTEM);
      manager.addForum(createDesignByContractForum(), SYSTEM);
      manager.addForum(createAspectOrientedProgrammingForum(), SYSTEM);
      manager.addForum(createStrutsStrategieEtDesignPatternsForum(), SYSTEM);

    } catch (PermissionException ex) {
      throw new IllegalStateException("Le systme devrait avoir le droit d'initialier le forumManager");
    }

    return manager;
  }

  private Forum createLetsStrutsItForum() throws PermissionException {
    Forum forum = new Forum("2100 - Let's Struts it !");
    forum.addMessage(new Message("Confrencier", "Emmanuel Boudrant"), SYSTEM);
    forum.addMessage(new Message("Description de la session 2100", letMessage), SYSTEM);
    return forum;
  }

  private Forum createRefactoringForum() throws PermissionException {
    Forum forum = new Forum("2102 - Refactoring");
    forum.addMessage(new Message("Confrencier", "Olivier Mangez"), SYSTEM);
    forum.addMessage(new Message("Description de la session Refactoring", refMessage), SYSTEM);
    return forum;
  }

  private Forum createAspectOrientedProgrammingForum() throws PermissionException {
    Forum forum = new Forum("2106 - Programmation oriente aspect");
    forum.addMessage(new Message("Confrenciers de la session 2106", "Emmanuel Boudrant et Olivier Mangez"), SYSTEM);
    forum.addMessage(new Message("Description de la session 2106", aopMessage), SYSTEM);
    return forum;
  }

  private Forum createDesignByContractForum() throws PermissionException {
    Forum forum = new Forum("2104 - Design by contract");
    forum.addMessage(new Message("Confrencier de la session Design by Contract", "Olivier Mangez"), SYSTEM);
    forum.addMessage(new Message("Description", dbcMessage), SYSTEM);
    return forum;
  }

  private Forum createStrutsStrategieEtDesignPatternsForum() throws PermissionException {
    Forum forum = new Forum("4104 - Struts : Stratgies et Design Patterns ");
    forum.addMessage(new Message("Confrencier", "Emmanuel Boudrant"), SYSTEM);
    forum.addMessage(new Message("Description de la session 2104", strMessage), SYSTEM);
    return forum;
  }
}