package com.borcon.aop.application;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

import java.util.List;

import com.borcon.aop.business.*;
import com.borcon.aop.security.*;
import com.borcon.aop.*;

/**
 * Borcon 2002 : 2106 - Programmation oriente aspect
 * @author Emmanuel Boudrant, Olivier Mangez
 *
 * Pour plus d'information :
 *   http://www.mangez.info
 *   http://www.mycgiserver.com/~eboudrant/borcon2002/
 */

public class MainFrame extends JFrame {

  private JPanel contentPane;

  private JMenuBar menuBar                      = new JMenuBar();
  private JMenu fileMenu                        = new JMenu();
  private JMenuItem fileExitMenu                = new JMenuItem();
  private JMenuItem fileChangeIdentityMenu      = new JMenuItem();
  private JMenu helpMenu                        = new JMenu();
  private JMenuItem helpAboutMenu               = new JMenuItem();
  private JLabel statusBar                      = new JLabel();
  private BorderLayout layout                   = new BorderLayout();
  private JSplitPane forumsSplitPane            = new JSplitPane();
  private JSplitPane messagesSplitPane          = new JSplitPane();
  private JScrollPane messageScrollPane         = new JScrollPane();

  private CollectionListModel forumsListModel   = new CollectionListModel();
  private JList forumsList                      = new JList(forumsListModel);
  private JLabel noForumsLabel                  = new JLabel("Vous n'avez accs  aucun forum");

  private CollectionListModel messagesListModel = new CollectionListModel();
  private JList messagesList                    = new JList(messagesListModel);
  private JLabel noMessagesLabel                = new JLabel();

  private ResourceCellRender resourceRender     = new ResourceCellRender();

  private JEditorPane messageEditorPane         = new JEditorPane();

  private ForumManager manager                  = null;
  private User user                             = null;

  public void showForums() {
    forumsListModel.setContent(manager.getForums(user));

    if(forumsListModel.getSize() > 0) {
      forumsSplitPane.add(forumsList, JSplitPane.LEFT);
    } else {
      forumsSplitPane.add(noForumsLabel, JSplitPane.LEFT);
    }
  }

  public void showMessages() {
    Forum forum = (Forum) forumsList.getSelectedValue();

    if(forum != null) {
      try {

        messagesListModel.setContent(forum.getMessages(user));

        if(messagesListModel.getSize() > 0) {
          messagesSplitPane.add(messagesList, JSplitPane.TOP);

        } else {
          noMessagesLabel.setText("Ce forum ne contient aucun message");
          messagesSplitPane.add(noMessagesLabel, JSplitPane.TOP);
        }

      } catch (PermissionException ex) {
        noMessagesLabel.setText("Vous n'avez pas accs  ce forum");
        messagesSplitPane.add(noMessagesLabel, JSplitPane.TOP);
      }
    } else {
      noMessagesLabel.setText("Veuillez slectioner un forum");
      messagesSplitPane.add(noMessagesLabel, JSplitPane.TOP);
    }
  }

  public void showMessageContent() {
    Message message = (Message) messagesList.getSelectedValue();

    if(message != null) {
      try {
        messageEditorPane.setText(message.getBody(user));
      } catch (PermissionException ex) {
        messageEditorPane.setText("<font color='red'>Vous n'tes pas autoris  lire ce message</font>");
      }
    } else {
      messageEditorPane.setText("<font color='red'>Slectionnez un message</font>");
    }
  }

  public void setUser(User user) {
    this.user = user;

    statusBar.setText(
        user == null
        ? "Utilisateur non identifi"
        : "Utilisateur : " + user.getId()
    );

    showForums();
    showMessages();
    showMessageContent();
  }

  private void selectedForumChanged(ListSelectionEvent e) {
    if (!e.getValueIsAdjusting()) {
      showMessages();
      showMessageContent();
    }
  }

  private void selectedMessageChanged(ListSelectionEvent e) {
    if (!e.getValueIsAdjusting()) {
      showMessageContent();
    }
  }

  //Construire le cadre
  public MainFrame(ForumManager manager) {

    this.manager = manager;

    enableEvents(AWTEvent.WINDOW_EVENT_MASK);

    try {
      jbInit();
    } catch(Exception e) {
      e.printStackTrace();
    }

    setUser(null);
  }

  //Initialiser le composant
  private void jbInit() throws Exception  {

    // Gnral

    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(layout);

    this.setSize(new Dimension(400, 300));
    this.setTitle("Lecteur de forums");

    noMessagesLabel.setBackground(Color.white);
    noMessagesLabel.setOpaque(true);

    noForumsLabel.setBackground(Color.white);
    noForumsLabel.setOpaque(true);

    // Menu

    fileMenu.setText("Action");
    messagesList.setMinimumSize(new Dimension(100, 100));
    messageEditorPane.setMinimumSize(new Dimension(100, 100));
    forumsList.setMinimumSize(new Dimension(200, 400));
    messageScrollPane.setMinimumSize(new Dimension(100, 100));
    menuBar.add(fileMenu);

    fileChangeIdentityMenu.setText("Changer d\'identit");
    fileChangeIdentityMenu.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        fileChangeIdentityMenu_actionPerformed(e);
      }
    });

    fileMenu.add(fileChangeIdentityMenu);

    fileExitMenu.setText("Quitter");
    fileExitMenu.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        fileExitMenu_actionPerformed(e);
      }
    });
    fileMenu.add(fileExitMenu);

    helpMenu.setText("Aide");
    menuBar.add(helpMenu);

    helpAboutMenu.setText("A propos");
    helpAboutMenu.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        helpAboutMenu_actionPerformed(e);
      }
    });
    helpMenu.add(helpAboutMenu);

    this.setJMenuBar(menuBar);

    // Barre de statut

    statusBar.setText("");
    contentPane.add(statusBar, BorderLayout.SOUTH);

    // Divers

    contentPane.add(forumsSplitPane, BorderLayout.CENTER);
    forumsSplitPane.add(messagesSplitPane, JSplitPane.RIGHT);
    messagesSplitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
    messagesSplitPane.add(messageScrollPane, JSplitPane.BOTTOM);
    messageScrollPane.getViewport().add(messageEditorPane, null);

    // Forums

    forumsList.setCellRenderer(resourceRender);
    forumsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

    forumsList.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        selectedForumChanged(e);
      }
    });

    // Messages

    messagesList.setCellRenderer(resourceRender);
    messagesList.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);

    messagesList.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        selectedMessageChanged(e);
      }
    });

    // Contenu d'un message

    messageEditorPane.setEditorKit(
        JEditorPane.createEditorKitForContentType("text/html")
    );
    messageEditorPane.setEditable(false);
    messageEditorPane.setText("");
  }

  //Opration Fichier | Quitter effectue
  public void fileExitMenu_actionPerformed(ActionEvent e) {
    System.exit(0);
  }

  //Opration Aide | A propos effectue
  public void helpAboutMenu_actionPerformed(ActionEvent e) {
    AboutDialog dlg = new AboutDialog(this);
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = getSize();
    Point loc = getLocation();
    dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.setModal(true);
    dlg.pack();
    dlg.show();
  }

  //Opration Aide | A propos effectue
  public void fileChangeIdentityMenu_actionPerformed(ActionEvent e) {
    ChangeIdentityDialog dlg = new ChangeIdentityDialog(this);
    Dimension dlgSize = dlg.getPreferredSize();
    Dimension frmSize = getSize();
    Point loc = getLocation();
    dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
    dlg.setModal(true);
    dlg.pack();
    dlg.show();
  }

  //Supplant, ainsi nous pouvons sortir quand la fentre est ferme
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      fileExitMenu_actionPerformed(null);
    }
  }
}